import { declare } from '@babel/helper-plugin-utils';
import syntaxRecordAndTuple from '@babel/plugin-syntax-record-and-tuple';
import { types } from '@babel/core';
import { isModule, addNamed } from '@babel/helper-module-imports';
import { OptionValidator } from '@babel/helper-validator-option';

const v = new OptionValidator("@babel/plugin-proposal-record-and-tuple");
var index = declare((api, options) => {
  api.assertVersion("8.0.0-alpha.5");
  const polyfillModuleName = v.validateStringOption("polyfillModuleName", options.polyfillModuleName, "@bloomberg/record-tuple-polyfill");
  const shouldImportPolyfill = v.validateBooleanOption("importPolyfill", options.importPolyfill, !!options.polyfillModuleName);
  const importCaches = new WeakMap();
  function getOr(map, key, getDefault) {
    let value = map.get(key);
    if (!value) map.set(key, value = getDefault());
    return value;
  }
  function getBuiltIn(name, programPath) {
    if (!shouldImportPolyfill) return types.identifier(name);
    if (!programPath) {
      throw new Error("Internal error: unable to find the Program node.");
    }
    const cacheKey = `${name}:${isModule(programPath)}`;
    const cache = getOr(importCaches, programPath.node, () => new Map());
    const localBindingName = getOr(cache, cacheKey, () => {
      return addNamed(programPath, name, polyfillModuleName, {
        importedInterop: "uncompiled"
      }).name;
    });
    return types.identifier(localBindingName);
  }
  return {
    name: "proposal-record-and-tuple",
    inherits: syntaxRecordAndTuple,
    visitor: {
      Program(path, state) {
        state.programPath = path;
      },
      RecordExpression(path, state) {
        const record = getBuiltIn("Record", state.programPath);
        const object = types.objectExpression(path.node.properties);
        const wrapped = types.callExpression(record, [object]);
        path.replaceWith(wrapped);
      },
      TupleExpression(path, state) {
        const tuple = getBuiltIn("Tuple", state.programPath);
        const wrapped = types.callExpression(tuple, path.node.elements);
        path.replaceWith(wrapped);
      }
    }
  };
});

export { index as default };
//# sourceMappingURL=index.js.map
